﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;

using MindFusion.Diagramming.Silverlight;


namespace DragDrop
{
	public partial class MainPage : UserControl
	{
		public MainPage()
		{
			InitializeComponent();

			diagram.MouseMove += new MouseEventHandler(diagram_MouseMove);
			diagram.MouseLeftButtonUp += new MouseButtonEventHandler(diagram_MouseLeftButtonUp);
			diagram.MouseLeftButtonDown += new MouseButtonEventHandler(diagram_MouseLeftButtonDown);

			// No special shape
			//diagram.DefaultShape =
			//    new MindFusion.Diagramming.Silverlight.Shape(
			//        new ElementTemplate[]
			//        {
			//            new LineTemplate(0, 0, 100, 0),
			//            new LineTemplate(100, 0, 100, 100),
			//            new LineTemplate(100, 100, 0, 100),
			//            new LineTemplate(0, 100, 0, 0),
			//        },
			//        null,
			//        FillRule.EvenOdd, "draggableShape",
			//        new ShapeDecoration[]
			//        {
			//            new ShapeDecoration(
			//                new ElementTemplate[]
			//                {
			//                    new LineTemplate(2, 2, 20, 2),
			//                    new LineTemplate(20, 2, 20, 20),
			//                    new LineTemplate(20, 20, 2, 20),
			//                    new LineTemplate(2, 20, 2, 2),
			//                }, Brushes.Blue, FillRule.EvenOdd, new Pen(Brushes.Black, 1))
			//        });

			const double size = 50;
			double x = 0;
			double y = 0;
			foreach (string shapeId in Shape.Shapes.Keys)
			{
				var node = diagram.Factory.CreateShapeNode(x, y, 50, 50);
				node.Shape = Shape.Shapes[shapeId];
				x += size + 5;
				if (x > 300)
				{
					y += size + 5;
					x = 0;
				}
			}
		}



		void diagram_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
		{
			ShapeNode node = diagram.GetNodeAt(e.GetPosition(diagram)) as ShapeNode;
			if (node == null)
				return;

			// Check if the mouse is over the node's decoration
			Rect r = node.GetBounds();
			Rect d = r;

			// We are starting drag&drop operation regardless of the click spot
			//d.X = r.X + r.Width * 0.02;
			//d.Y = r.Y + r.Height * 0.02;
			//d.Width = r.Width * 0.18;
			//d.Height = r.Height * 0.18;

			if (!d.Contains(e.GetPosition(diagram)))
				return;

			// Start draging
			dragging = true;
			info = new DragDropInfo() { ShapeId = node.Shape.Id };
			diagram.CaptureMouse();

			mouse = e.GetPosition(ddplane);

			test.Width = node.Width;
			test.Height = node.Height;
			Canvas.SetLeft(test, mouse.X - test.Width / 2);
			Canvas.SetTop(test, mouse.Y - test.Height / 2);
			test.Opacity = 1;
			test.Visibility = Visibility.Visible;
		}

		void diagram_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
		{
			if (dragging)
			{
				diagram.ReleaseMouseCapture();
				dragging = false;

				DoubleAnimation a = new DoubleAnimation()
				{
					To = 0,
					Duration = TimeSpan.FromSeconds(0.25),
				};

				Storyboard.SetTarget(a, test);
				Storyboard.SetTargetProperty(a, new PropertyPath("UIElement.Opacity"));
				Storyboard s = new Storyboard();
				s.Children.Add(a);
				s.Completed += (s1, e1) =>
					{
						test.Visibility = Visibility.Collapsed;
					};
				s.Begin();

				// hit test for ddtarget
				List<UIElement> hits = System.Windows.Media.VisualTreeHelper.FindElementsInHostCoordinates(
					e.GetPosition(LayoutRoot),
					LayoutRoot) as List<UIElement>;

				if (hits.Contains(test))
					hits.Remove(test);
				if (hits.Contains(ddplane))
					hits.Remove(ddplane);

				if (hits.Contains(ddtarget))
				{
					Point pos = e.GetPosition(ddtarget);
					var node = ddtarget.Factory.CreateShapeNode(pos.X - test.Width / 2,
						pos.Y - test.Height / 2, test.Width, test.Height);
					node.Shape = Shape.FromId(info.ShapeId);
				}
			}
		}

		void diagram_MouseMove(object sender, MouseEventArgs e)
		{
			if (dragging)
			{
				diagram.Behavior = Behavior.DoNothing;
				diagram.Behavior = Behavior.LinkShapes;

				Point newMouse = e.GetPosition(ddplane);

				double dx = newMouse.X - mouse.X;
				double dy = newMouse.Y - mouse.Y;

				Canvas.SetLeft(test, Canvas.GetLeft(test) + dx);
				Canvas.SetTop(test, Canvas.GetTop(test) + dy);

				mouse = newMouse;
			}
		}

		private bool dragging = false;
		private Point mouse = new Point();
		private DragDropInfo info;

		private void diagram_LinkCreating(object sender, LinkValidationEventArgs e)
		{
			if (dragging)
				e.CancelDrag();
		}
	}

	public class DragDropInfo
	{
		public string ShapeId
		{
			get;
			set;
		}
	}
}